@@ -1,9 +1,8 @@
-import sys
 import warnings
 
 from numbers_parser.file import read_numbers_file
+from numbers_parser.constants import PACKAGE_ID
 from numbers_parser.iwafile import create_iwa_segment, copy_object_to_iwa_file, IWAFile
-from numbers_parser.generated import TSPMessages_pb2 as TSPMessages
 
 
 class ItemsList:
@@ -62,8 +61,8 @@ class ObjectStore:
 
     def _new_message_id(self):
         """Return the next available message ID for object creation"""
-        max_id = max(self._objects.keys())
-        return max_id + 1
+        self._objects[PACKAGE_ID].last_object_identifier += 1
+        return self._objects[PACKAGE_ID].last_object_identifier
 
     def mark_as_dirty(self, obj_id: int):
         self._dirty[obj_id] = True
@@ -96,6 +95,9 @@ class ObjectStore:
     def update_dirty_objects(self):
         """Copy the protobuf messages from any updated object to the cached
         version in the file store so this can be saved to a new document"""
+        # TODO: Revisit which objects are dirty and only re-save those
+        for obj_id in self._objects.keys():
+            self.mark_as_dirty(obj_id)
         for obj_id in self._dirty.keys():
             copy_object_to_iwa_file(
                 self._file_store[self._object_to_filename_map[obj_id]],
@@ -103,33 +105,6 @@ class ObjectStore:
                 obj_id,
             )
 
-    def deep_print(self, obj, indent=0, file=sys.stdout):  # pragma: no cover
-        for descriptor in obj.DESCRIPTOR.fields:
-            value = getattr(obj, descriptor.name)
-            if isinstance(value, TSPMessages.Reference):
-                id = value.identifier
-                name = descriptor.full_name
-                if id != 0:
-                    type_name = self[id].DESCRIPTOR.full_name
-                    print("  " * indent + f"{name} -> #{id} ({type_name})", file=file)
-                    self.deep_print(self[id], indent=indent + 1, file=file)
-                else:
-                    print("  " * indent + f"{name} -> #{id}", file=file)
-            elif descriptor.type == descriptor.TYPE_MESSAGE:
-                if descriptor.label == descriptor.LABEL_REPEATED:
-                    map(lambda x: self.deep_print(x, indent + 1), value)
-                else:
-                    print("  " * indent + f"{descriptor.full_name}", file=file)
-                    self.deep_print(value, indent=indent + 1, file=file)
-            elif descriptor.type == descriptor.TYPE_ENUM:
-                if type(value).__name__ == "RepeatedScalarContainer":
-                    enum_str = [descriptor.enum_type.values[x].name for x in value]
-                else:
-                    enum_str = descriptor.enum_type.values[value - 1].name
-                print("  " * indent + f"{descriptor.full_name}, {enum_str}", file=file)
-            else:
-                print("  " * indent + f"{descriptor.full_name} {value}", file=file)
-
     def _store_object(self, identifier, obj, filename):
         self._objects[identifier] = obj
         self._object_to_filename_map[identifier] = filename
@@ -147,6 +122,7 @@ class ObjectStore:
     def __len__(self) -> int:
         return len(self._objects)
 
+    # Don't cache: new tables and sheets can be added at runtime
     def find_refs(self, ref_name) -> list:
         refs = [k for k, v in self._objects.items() if type(v).__name__ == ref_name]
         return refs
